' iTunesPlaylistExportToVolvo.vbs [UNIX]
' For UNIX create a Windows shortcut that looks like "D:\Playlist Development\iTunesPlaylistExportToVolvo5.vbs" UNIX
'
' THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
' IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
' FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
' THE AUTHOR OR ANY OTHER CONTRIBUTOR BE LIABLE FOR ANY CLAIM, DAMAGES 
' OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
' ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
' DEALINGS IN THE SOFTWARE.
'
' (c) Paul Simmonds, 2019;  www.simmonds.org.uk
' Some iTunes export code from an original work by
' (C) Dirk Hedderich, 2009; www.verheddert.de/itunes Version 1.0, 2009-02-01
' Status box code adapted from (Meticulix) by combining formulas taught by Denis St-Pierre's '"Progress Message" and SimplyCoded's "Part 24: Replace Lines" Tutorial

Option Explicit

' Makes the object a Public object (Critical!)
Dim objProgressMsg

Dim strWindowTitle : strWindowTitle = "iTunes Playlist to USB - STATUS (Do NOT Close]"

'On Error Resume Next

Dim driveLetter
Dim driveName
Dim iTunes
Dim targetDrive
Dim CONST_FilePrefix
Dim CONST_Silent
Dim BtnCode
Dim WshShell
Dim track
Dim myItem
Dim objFSO
Dim Drives
Dim strMessage
Dim textFile
Dim colDrives
Dim colSources
Dim colPlaylists
Dim objDrive
Dim objSource
Dim objPlaylist
Dim trackLocation
Dim itemLocation
Dim result
Dim skipIfExist
Dim unixFileLocation
Dim destinationFile
Dim destinationFolder
Dim destinationFolderArray
Dim strDirBuild
Dim i
Dim copyResult
Dim fsUnix: fsUnix = False

Set objFSO = CreateObject("Scripting.FileSystemObject")
Set colDrives = objFSO.Drives
Set WshShell = WScript.CreateObject("WScript.Shell")

ProgressMsg "Now starting iTunes to look for Playlists, please wait . . . ", strWindowTitle

Set iTunes = CreateObject ("iTunes.Application")
Set colSources = iTunes.Sources

'Check the first argument 

If Wscript.Arguments.Count > 0 Then
    If Wscript.Arguments(0) = "UNIX" Then
		fsUnix = True
		ProgressMsg "UNIX file seperators set, please wait . . . ", strWindowTitle
		MsgBox("m3U will be written in UNIX format - with / file seperators")
	End if
End If


' Give the user an initial progress
ProgressMsg "Looking for a suitable USB Drive, Please wait.", strWindowTitle

On Error Resume Next
For Each objDrive in colDrives
	ProgressMsg "Checking drive " & objDrive & " . . . Please wait!", strWindowTitle
	if (objDrive.DriveType = 1) AND (objDrive.IsReady = TRUE) Then
		if ((InStr(1, objDrive.VolumeName,"VOLVO MUSIC",1) > 0) OR (InStr(1, objDrive.VolumeName,"CAR MUSIC",1)) > 0) Then
			driveLetter = objDrive.DriveLetter
			driveName = objDrive.VolumeName
		end if
	end if
Next
On Error Goto 0     ' Normal Error Handling

if driveLetter <> "" Then

	ProgressMsg driveName & " found in Drive: " & driveLetter & ":", strWindowTitle
	result = MsgBox (driveName & " found in Drive: " & driveLetter & ":" & vbCrLf & "Proceed using this drive?", vbYesNo, "Write music & playlists")
	Select Case result
	Case vbYes
	
		result = MsgBox ("If the file already exists, do you want to" & vbCrLf & "skip writing the track to the USB?", vbYesNo, "Skip writing")
		Select Case result
		Case vbYes
			skipIfExist = True
		Case vbNo
			skipIfExist = False
		End Select
		
		targetDrive = driveLetter & ":\"

		For Each objSource in colSources
			'Wscript.Echo objSource.Name
			Set colPlaylists = objSource.Playlists
			For Each objPlaylist in colPlaylists
			
				If objPlaylist.Kind=2 then 
				
					If objPlaylist.SpecialKind=0 then 
				  
						If objPlaylist.Smart=False then
						
							'Start of individual Playlist loop
							result = MsgBox ("The Playlist '" & objPlaylist.Name & "' will be written to " & targetDrive & objPlaylist.Name & ".m3u" & vbCrLf & "Do you want to write this playlist?", vbYesNo, "Write music & playlists")
							if result = vbYes Then
							
								if objPlaylist.Tracks.Count < 1000 Then
							
									Set textFile = objFSO.CreateTextFile(targetDrive + objPlaylist.Name + ".m3u", true, false) ' Options set to: "Over-write" and "Save as ASCII"

									'Write the Header
									textFile.WriteLine("#EXTM3U")

									' Loop over all Tracks of this Playlist
									for track = 1 to objPlaylist.Tracks.Count
										On Error Resume Next    ' Catch errors using iTunes items
										' Get item attributes
										do
											itemRead = True
											Err.Clear
											Set myItem = objPlaylist.Tracks.Item(track)
											If Err.Number = 0 Then itemLocation = myItem.Location End If
											
											'to get file path without drive letter, assuming drive letters are c:, d:, etc
											trackLocation = Split(itemLocation,":")
											'Format should mirror the iTunes location, but in DOS format, without the leading slash =  _A\AC-DC\AC-DC_-_Spellbound.mp3
											If fsUnix = True Then
												unixFileLocation = Replace(trackLocation(1), "\", "/")
											Else
												unixFileLocation = trackLocation(1)
											End if
											unixFileLocation = Mid(unixFileLocation,2)
											'unixFileLocation = track & " " & unixFileLocation
											'We are going to move this down and make it conditional on a sucessful copy (or file is already there)
											'textFile.WriteLine(unixFileLocation)														
									
											If Err.Number <> 0 Then
												'textFile.WriteLine("Error accessing iTunes! - " & myItem.Name)
												'textFile.WriteLine("Error accessing iTunes! - " & myItem.Name)
												itemRead = False  				
												BtnCode = WshShell.Popup("Error accessing iTunes! (" + myItem.Name + ")" + vbCrLf + "Please close all iTunes dialogues." + vbCrLf + vbCrLf + "Try again? Press Cancel to leave program.", 0, "iTunes Export Playlist to Folder", 5 + 16)
												If BtnCode = 2 then
													'textFile.WriteLine("Program aborted.")
													WScript.Quit()
												End If

											End If  
											
											
											'here is where we copy the file
											destinationFile = driveLetter & ":" & trackLocation(1)
											
											'Check to see if the file already exists in the destination folder OR we want to overwrite anyway
											If (objFSO.FileExists(destinationFile) = False) OR (skipIfExist = False) Then
											
												'Check to see if the file is read-only
												If objFSO.GetFile(destinationFile).Attributes And 1 Then  
													'The file exists and is read-only.
													'Remove the read-only attribute
													objFSO.GetFile(destinationFile).Attributes = objFSO.GetFile(destinationFile).Attributes - 1
												End If
																							
												destinationFolder = Left(destinationFile, InStrRev(destinationFile, "\"))
												If Not objFSO.FolderExists(destinationFolder) Then
												
													destinationFolderArray = split(destinationFolder, "\")
													strDirBuild = destinationFolderArray(0)

													' Check each (sub)folder and create it if it doesn't exist
													For i = 1 to UBound(destinationFolderArray)
														strDirBuild = strDirBuild & "\" & destinationFolderArray(i)

														'MsgBox(strDirBuild & " : " & destinationFolderArray(i) & " : " & i & " of " & UBound(destinationFolderArray))
														
														If Not objFSO.FolderExists( strDirBuild ) Then
															objFSO.CreateFolder strDirBuild
															'MsgBox("Make Directory: " & destinationFolderArray(i))
														End if
													Next
												
												End If

												'MsgBox("Actual PC Location = " & itemLocation & vbCrLf &"Destination = " & destinationFile & vbCrLf &"Desination Folder = " & destinationFolder & vbCrLf & "m3u text location = " & unixFileLocation)
												'It is safe to copy file to this folder.
												'MsgBox("Actual PC Location = " & itemLocation & vbCrLf &"Destination = " & destinationFile & vbCrLf &"Desination Folder = " & destinationFolder & vbCrLf & "m3u text location = " & unixFileLocation)
												ProgressMsg "Track " & track & " of " & objPlaylist.Tracks.Count & " - " & objPlaylist.Name & ".m3u [Copy file to USB]", strWindowTitle
												copyResult = objFSO.CopyFile(itemLocation, destinationFile, True)
												
												'If the file sucessfully copied then it's safe to write the file
												ProgressMsg "Track " & track & " of " & objPlaylist.Tracks.Count & " - " & objPlaylist.Name & ".m3u [Write track to m3u]", strWindowTitle
												if (copyResult = 0) Then  textFile.WriteLine(unixFileLocation)
												
											Else
											
												'If the file already exists and we skipped then it's safe to write the file
												ProgressMsg "Track " & track & " of " & objPlaylist.Tracks.Count & " - " & objPlaylist.Name & ".m3u [Write track to m3u]", strWindowTitle
												textFile.WriteLine(unixFileLocation)
												
											End If
											
										Loop Until itemRead = True
														  
									  ' Read all item attributes, now do something ;-)  
									  On Error Goto 0     ' Normal Error Handling

									Next
									
								Else
								
									MsgBox ("This playlist has more than 1000 tracks and will not work in the vehicle")
								
								End if
								
							End If
							
						End If
					
					End If
				  
				End If
				
			Next
			
		Next

		textFile.Close()
		ProgressMsg "Done", "iTunes Playlist to USB - STATUS"
		WScript.Echo "Click [OK] to close program!"
		'WScript.Quit

	Case vbNo
		MsgBox("User aborted the utility")
	End Select	

Else
	Wscript.echo "No USB called 'VOLVO MUSIC' or 'CAR MUSIC' found"
End If

'final Clean up
Set objFSO = Nothing
'Set wshMsgShell = Nothing
'Set objMsgFSO   = Nothing
WScript.Quit

Function ProgressMsg(strMessage, strWindowTitle )
' Written by Denis St-Pierre
' Displays a progress message box that the originating script can kill in both 2k and XP
' If StrMessage is blank, take down previous progress message box
' Using 4096 in Msgbox below makes the progress message float on top of things
' CAVEAT: You must have Dim ObjProgressMsg at the top of your script for this to work as described
    Dim strTEMP
	Dim strTempVBS
	Dim objTempMessage
	Dim wshMsgShell
	Dim objMsgFSO
	
	Set wshMsgShell = CreateObject( "WScript.Shell" )
    strTEMP = WshMsgShell.ExpandEnvironmentStrings( "%TEMP%" )
    If strMessage = "" Then
        ' Disable Error Checking in case objProgressMsg doesn't exists yet
        On Error Resume Next
        ' Kill ProgressMsg
        objProgressMsg.Terminate( )
        ' Re-enable Error Checking
        On Error Goto 0
        Exit Function
    End If
    Set objMsgFSO = CreateObject("Scripting.FileSystemObject")
    strTempVBS = strTEMP + "\" & "Message.vbs"     'Control File for reboot

    ' Create Message.vbs, True=overwrite
    Set objTempMessage = objMsgFSO.CreateTextFile( strTempVBS, True )
    objTempMessage.WriteLine( "MsgBox""" & strMessage & """, 4096, """ & strWindowTitle & """" )
    objTempMessage.Close

    ' Disable Error Checking in case objProgressMsg doesn't exists yet
    On Error Resume Next
    ' Kills the Previous ProgressMsg
    objProgressMsg.Terminate( )
    ' Re-enable Error Checking
    On Error Goto 0

    ' Trigger objProgressMsg and keep an object on it
    Set objProgressMsg = WshShell.Exec( "%windir%\system32\wscript.exe " & strTempVBS )

    Set wshMsgShell = Nothing
    Set objMsgFSO   = Nothing
End Function